window.Gleam.initSearch([{"doc":"lifeguard","title":"lifeguard","content":"# 🛟 Lifeguard\n\n[![Package Version](https://img.shields.io/hexpm/v/lifeguard)](https://hex.pm/packages/lifeguard)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/lifeguard/)\n\nLifeguard a generic actor pool for Gleam. It can be used to interface with a pool of\nsupervised actors without having to manage their lifecycles yourself.\n\n## Installation\n\n```sh\ngleam add lifeguard@1\n```\n\n## Usage\n\n```gleam\nimport lifeguard\nimport fake_db\n\npub fn main() {\n  // Create a pool of 10 connections to some fictional database.\n  let assert Ok(pool) =\n    lifeguard.new(\n      lifeguard.Spec(\n        init_timeout: 1000,\n        init: fn(selector) { actor.Ready(state: fake_db.get_conn(), selector:) },\n        loop: fn(msg, state) {\n          case msg {\n            // Logic here...\n          }\n        },\n      )\n    )\n    |&gt; lifeguard.with_size(10)\n    |&gt; lifeguard.start(1000)\n\n  // Send a message to the pool\n  let assert Ok(Nil) =\n    lifeguard.send(pool, fake_db.Ping, 1000)\n\n  // Send a message to the pool and wait for a response\n  let assert Ok(fake_db.Pong) =\n    lifeguard.call(pool, fake_db.Query(_, &quot;select 1&quot;), 1000, 1000)\n\n  // Shut down the pool\n  lifeguard.shutdown(pool)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/lifeguard&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"lifeguard","title":"ApplyError","content":"pub type ApplyError {\n  NoResourcesAvailable\n  CheckOutTimeout\n  WorkerCallTimeout\n  WorkerCrashed(dynamic.Dynamic)\n}\n An error returned when failing to use a pooled worker.\nNoResourcesAvailable\n\n\nCheckOutTimeout\n\n\nWorkerCallTimeout\n\n\nWorkerCrashed(dynamic.Dynamic)\n\n\nSynonyms:\nlifeguard.ApplyError\nlifeguard ApplyError","url":"lifeguard.html#ApplyError"},{"doc":"lifeguard","title":"CheckoutStrategy","content":"pub type CheckoutStrategy {\n  FIFO\n  LIFO\n}\n The strategy used to check out a resource from the pool.\nFIFO\n\n\nLIFO\n\n\nSynonyms:\nlifeguard.CheckoutStrategy\nlifeguard CheckoutStrategy","url":"lifeguard.html#CheckoutStrategy"},{"doc":"lifeguard","title":"Pool","content":"pub opaque type Pool(msg)\n The interface for interacting with a pool of workers in Lifeguard.\n\nSynonyms:\nlifeguard.Pool\nlifeguard Pool","url":"lifeguard.html#Pool"},{"doc":"lifeguard","title":"PoolConfig","content":"pub opaque type PoolConfig(state, msg)\n Configuration for a [`Pool`](#Pool).\n\nSynonyms:\nlifeguard.PoolConfig\nlifeguard PoolConfig","url":"lifeguard.html#PoolConfig"},{"doc":"lifeguard","title":"Spec","content":"pub type Spec(state, msg) {\n  Spec(\n    init: fn(process.Selector(msg)) -&gt;\n      actor.InitResult(state, msg),\n    init_timeout: Int,\n    loop: fn(msg, state) -&gt; actor.Next(msg, state),\n  )\n}\n An actor spec for workers in a pool. Similar to [`actor.Spec`](https://hexdocs.pm/gleam_otp/gleam/otp/actor.html#Spec),\n but this provides the initial selector for the actor. It will select on its own\n subject by default, using `function.identity` to pass the message straight through.\n\n For clarity, it will be used as follows by Lifeguard:\n\n \n actor.Spec(init_timeout: spec.init_timeout, loop: spec.loop, init: fn() {\n   // Check in the worker\n   let self = process.new_subject()\n   process.send(pool_subject, Register(self)) // Register the worker with the pool\n\n   let selector =\n     process.new_selector()\n     |&gt; process.selecting(self, function.identity)\n\n   spec.init(selector)\n })\n \nSpec(\n  init: fn(process.Selector(msg)) -&gt; actor.InitResult(state, msg),\n  init_timeout: Int,\n  loop: fn(msg, state) -&gt; actor.Next(msg, state),\n)\n\n\nSynonyms:\nlifeguard.Spec\nlifeguard Spec","url":"lifeguard.html#Spec"},{"doc":"lifeguard","title":"StartError","content":"pub type StartError {\n  PoolActorStartError(actor.StartError)\n  WorkerStartError(actor.StartError)\n  PoolSupervisorStartError(dynamic.Dynamic)\n  WorkerSupervisorStartError(dynamic.Dynamic)\n}\n An error returned when creating a [`Pool`](#Pool).\nPoolActorStartError(actor.StartError)\n\n\nWorkerStartError(actor.StartError)\n\n\nPoolSupervisorStartError(dynamic.Dynamic)\n\n\nWorkerSupervisorStartError(dynamic.Dynamic)\n\n\nSynonyms:\nlifeguard.StartError\nlifeguard StartError","url":"lifeguard.html#StartError"},{"doc":"lifeguard","title":"call","content":"pub fn call(\n  pool pool: Pool(a),\n  msg msg: fn(Subject(b)) -&gt; a,\n  checkout_timeout checkout_timeout: Int,\n  call_timeout call_timeout: Int,\n) -&gt; Result(b, ApplyError)\n Send a message to a pooled actor and wait for a response. Equivalent to `process.call`\n using a pooled actor.\nSynonyms:\nlifeguard.call\nlifeguard call","url":"lifeguard.html#call"},{"doc":"lifeguard","title":"new","content":"pub fn new(spec spec: Spec(a, b)) -&gt; PoolConfig(a, b)\n Create a new [`PoolConfig`](#PoolConfig) for creating a pool of actors.\n\n \n import lifeguard\n\n pub fn main() {\n   // Create a pool of 10 actors that do nothing.\n   let assert Ok(pool) =\n     lifeguard.new(\n       lifeguard.Spec(\n         init_timeout: 1000,\n         init: fn(selector) { actor.Ready(state: Nil, selector:) },\n         loop: fn(msg, state) { actor.continue(state) },\n       )\n     )\n     |&gt; lifeguard.with_size(10)\n     |&gt; lifeguard.start(1000)\n }\n \n\n ### Default values\n\n | Config | Default |\n |--------|---------|\n | `size`   | 10      |\n | `checkout_strategy` | `FIFO` |\nSynonyms:\nlifeguard.new\nlifeguard new","url":"lifeguard.html#new"},{"doc":"lifeguard","title":"send","content":"pub fn send(\n  pool pool: Pool(a),\n  msg msg: a,\n  checkout_timeout checkout_timeout: Int,\n) -&gt; Result(Nil, ApplyError)\n Send a message to a pooled actor. Equivalent to `process.send` using a pooled actor.\nSynonyms:\nlifeguard.send\nlifeguard send","url":"lifeguard.html#send"},{"doc":"lifeguard","title":"shutdown","content":"pub fn shutdown(pool pool: Pool(a)) -&gt; Nil\n Shut down a pool and all its workers.\nSynonyms:\nlifeguard.shutdown\nlifeguard shutdown","url":"lifeguard.html#shutdown"},{"doc":"lifeguard","title":"start","content":"pub fn start(\n  config pool_config: PoolConfig(a, b),\n  timeout init_timeout: Int,\n) -&gt; Result(Pool(b), StartError)\n Start a pool supervision tree using the given [`PoolConfig`](#PoolConfig) and return a\n [`Pool`](#Pool).\nSynonyms:\nlifeguard.start\nlifeguard start","url":"lifeguard.html#start"},{"doc":"lifeguard","title":"supervisor","content":"pub fn supervisor(pool pool: Pool(a)) -&gt; Pid\n Get the supervisor PID for a running pool.\nSynonyms:\nlifeguard.supervisor\nlifeguard supervisor","url":"lifeguard.html#supervisor"},{"doc":"lifeguard","title":"with_checkout_strategy","content":"pub fn with_checkout_strategy(\n  config pool_config: PoolConfig(a, b),\n  strategy checkout_strategy: CheckoutStrategy,\n) -&gt; PoolConfig(a, b)\n Set the order in which actors are checked out from the pool. Defaults to `FIFO`.\nSynonyms:\nlifeguard.with_checkout_strategy\nlifeguard with_checkout_strategy","url":"lifeguard.html#with_checkout_strategy"},{"doc":"lifeguard","title":"with_size","content":"pub fn with_size(\n  config pool_config: PoolConfig(a, b),\n  size size: Int,\n) -&gt; PoolConfig(a, b)\n Set the number of actors in the pool. Defaults to 10.\nSynonyms:\nlifeguard.with_size\nlifeguard with_size","url":"lifeguard.html#with_size"},{"doc":"lifeguard","title":"lifeguard","content":"","url":"lifeguard.html"}]);